unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, Menus, ExtCtrls, Math, ComCtrls;

type
  TRoad = record
    FromCity, ToCity: String;
    Length: Integer;
    AvSpeed: Integer;
  end;

  TCity = record
    Name: String;
    X, Y: Integer;
  end;

  TFuel = record
    AvSpeed: Integer;
    Amount: Single;
  end;

  TMap = class
    Name: String;
    City: Array of TCity;
    Road: Array of TRoad;
    Fuel: Array of TFuel;

    MaxX, MinX, MaxY, MinY: Integer;
    procedure Load(FileName: String);
    function FindCity(Name: String): Integer;
    function GetCityFromPos(X, Y: Integer): Integer;
    function FindRoad(FromCity, ToCity: Integer; R: Integer): Integer;
  end;

  TForm1 = class(TForm)
    MainMenu1: TMainMenu;
    NastMapu1: TMenuItem;
    Nast1: TMenuItem;
    N1: TMenuItem;
    Konec1: TMenuItem;
    OpenDialog1: TOpenDialog;
    MapView: TPaintBox;
    Zobrazit1: TMenuItem;
    Cesty1: TMenuItem;
    StatusBar1: TStatusBar;
    Vyhledatcestu1: TMenuItem;
    procedure Konec1Click(Sender: TObject);
    procedure Nast1Click(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure MapViewPaint(Sender: TObject);
    procedure Cesty1Click(Sender: TObject);
    procedure MapViewMouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
    procedure MapViewMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure Vyhledatcestu1Click(Sender: TObject);
  private
    { Private declarations }
  public
    Map: TMap;
  end;

var
  Form1: TForm1;
  
  Settings: TFormatSettings;
  CenterX, CenterY: Integer;
  Zoom: Single;
  Loading: Boolean;

implementation

uses Unit2, Unit3;

{$R *.dfm}

function ExtractValue(const S: String): String;
begin
  Result := Copy(S, Succ(AnsiPos('>', S)), AnsiPos('</', S) - Succ(AnsiPos('>', S)))
end;

function BlendColor(Color1, Color2: TColor; A: Byte): TColor;
begin
  Result := RGB((GetRValue(ColorToRGB(Color1)) * A + GetRValue(ColorToRGB(Color2)) * ($FF - A)) div $FF, (GetGValue(ColorToRGB(Color1)) * A + GetGValue(ColorToRGB(Color2)) * ($FF - A)) div $FF, (GetBValue(ColorToRGB(Color1)) * A + GetBValue(ColorToRGB(Color2)) * ($FF - A)) div $FF);
end;

function TMap.FindRoad(FromCity, ToCity: Integer; R: Integer): Integer;
var I: Integer;
begin
  for I := Succ(R) to High(Road) do
  begin
    if AnsiSameText(City[FromCity].Name, Road[I].FromCity) then
    begin
      Result := I;
      Exit;
    end;
    if AnsiSameText(City[FromCity].Name, Road[I].ToCity) then
    begin
      Result := I;
      Exit;
    end;
  end;
  Result := -1;
end;

function TMap.GetCityFromPos(X, Y: Integer): Integer;
var I: Integer;
begin
  for I := 0 to High(City) do
  begin
    if Sqr(City[I].X - X) + Sqr(City[I].Y - Y) < 50 then
    begin
      Result := I;
      Exit;
    end;
  end;
  Result := -1;
end;

function TMap.FindCity(Name: String): Integer;
var I: Integer;
begin
  for I := 0 to High(City) do
  begin
    if AnsiSameText(City[I].Name, Name) then
    begin
      Result := I;
      Exit;
    end;
  end;
  Result := -1;
end;

procedure TMap.Load(FileName: String);
var
  F: TextFile;
  Line, S, Tag: String;

  procedure LoadCity;
  begin
    repeat
      ReadLn(F, Line);
      S := Trim(Line);
      if AnsiSameText(S, '<mesto>') then
      begin
        SetLength(City, Succ(Length(City)));
        repeat
          ReadLn(F, Line);
          S := Trim(Line);
          Tag := Copy(Copy(S, AnsiPos('<', S), MaxInt), 1, AnsiPos('>', S));
          if AnsiSameText(Tag, '<nazev>') then
            City[High(City)].Name := ExtractValue(S);
          if AnsiSameText(Tag, '<polohaX>') then
          begin
            City[High(City)].X := StrToInt(ExtractValue(S));
            if City[High(City)].X > MaxX then MaxX := City[High(City)].X;
            if City[High(City)].X < MinX then MinX := City[High(City)].X;
          end;
          if AnsiSameText(Tag, '<polohaY>') then
          begin
            City[High(City)].Y := StrToInt(ExtractValue(S));
            if City[High(City)].Y > MaxY then MaxY := City[High(City)].Y;
            if City[High(City)].Y < MinY then MinY := City[High(City)].Y;
          end;
        until AnsiSameText(S, '</mesto>');
      end;
    until AnsiSameText(S, '</mesta>');
  end;

  procedure LoadRoad;
  begin
    repeat
      ReadLn(F, Line);
      S := Trim(Line);
      if AnsiSameText(S, '<cesta>') then
      begin
        SetLength(Road, Succ(Length(Road)));
        repeat
          ReadLn(F, Line);
          S := Trim(Line);
          Tag := Copy(Copy(S, AnsiPos('<', S), MaxInt), 1, AnsiPos('>', S));
          if AnsiSameText(Tag, '<z>') then
            Road[High(Road)].FromCity := ExtractValue(S);
          if AnsiSameText(Tag, '<do>') then
            Road[High(Road)].ToCity := ExtractValue(S);
          if AnsiSameText(Tag, '<vzdalenost>') then
            Road[High(Road)].Length := StrToInt(ExtractValue(S));
          if AnsiSameText(Tag, '<prumernaRychlost>') then
            Road[High(Road)].AvSpeed := StrToInt(ExtractValue(S));
        until AnsiSameText(S, '</cesta>');
      end;
    until AnsiSameText(S, '</cesty>');
  end;

  procedure LoadFuel;
  begin
    repeat
      ReadLn(F, Line);
      S := Trim(Line);
      if AnsiSameText(S, '<spotreba>') then
      begin
        SetLength(Fuel, Succ(Length(Fuel)));
        repeat
          ReadLn(F, Line);
          S := Trim(Line);
          Tag := Copy(Copy(S, AnsiPos('<', S), MaxInt), 1, AnsiPos('>', S));
          if AnsiSameText(Tag, '<rychlost>') then
            Fuel[High(Fuel)].AvSpeed := StrToInt(ExtractValue(S));
          if AnsiSameText(Tag, '<litru>') then
            Fuel[High(Fuel)].Amount := StrToFloat(ExtractValue(S), Settings);
        until AnsiSameText(S, '</spotreba>');
      end;
    until AnsiSameText(S, '</spotreby>');
  end;

begin
  Name := ExtractFileName(FileName);
  SetLength(City, 0);
  SetLength(Fuel, 0);
  MaxX := -MaxInt;
  MinX := MaxInt;
  MaxY := -MaxInt;
  MinY := MaxInt;

  AssignFile(F, FileName);
  Reset(F);
  try
    ReadLn(F, Line);
    if not AnsiSameText(Copy(Line, 1, 5), '<?xml') then
      raise Exception.CreateFmt('patn formt mapy v souboru %s.', [Name]);
    repeat
      ReadLn(F, Line);
    until AnsiSameText(Trim(Line), '<mapa>');
    repeat
      ReadLn(F, Line);
      S := Trim(Line);
      if AnsiSameText(S, '<mesta>') then LoadCity;
      if AnsiSameText(S, '<cesty>') then LoadRoad;
      if AnsiSameText(S, '<spotreby>') then LoadFuel;
    until EOF(F);
  finally
    CloseFile(F);
    Inc(MaxX, 60);
    Dec(MinX, 10);
    Inc(MaxY, 10);
    Dec(MinY, 10);
  end;
end;

procedure TForm1.Konec1Click(Sender: TObject);
begin
  Close;
end;

procedure TForm1.Nast1Click(Sender: TObject);
var I: Integer;
begin
  if OpenDialog1.Execute then
  begin
    Loading := True;
    Map.Load(OpenDialog1.FileName);
    Caption := 'Autoatlas - ' + Map.Name;

    with Form3 do
    begin
      ComboBox1.Items.Clear;
      ComboBox2.Items.Clear;

      for I := 0 to High(Map.City) do
      begin
        ComboBox1.Items.Add(Map.City[I].Name);
        ComboBox2.Items.Add(Map.City[I].Name);
      end;
    end;

    MapView.Invalidate;
    Loading := False;
  end;
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
  Map := TMap.Create;
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
  Map.Free;
end;

procedure TForm1.MapViewPaint(Sender: TObject);
var I, X, Y, X1, Y1, A1, A2: Integer;
begin
  CenterX := MapView.Width div 2;
  CenterY := MapView.Height div 2;

  if Abs(Map.MaxX - Map.MinX) > 0 then
  if Width / Abs(Map.MaxX - Map.MinX) > Height / Abs(Map.MaxY - Map.MinY) then
  begin
    Zoom := Height / (Map.MaxY - Map.MinY);
  end
  else
  begin
    Zoom := Width / (Map.MaxX - Map.MinX);
  end;

  with MapView.Canvas, Map do
  begin
    Brush.Color := clWhite;
    Pen.Color := clBlack;
    Font.Color := clNavy;
    FillRect(MapView.ClientRect);

    Brush.Style := bsClear;

    if Cesty1.Checked then 
    for I := 0 to High(Road) do
    begin
      A1 := FindCity(Road[I].FromCity);
      X := Round((City[A1].X - (Map.MaxX + Map.MinX) / 2) * Zoom);
      Y := Round((City[A1].Y - (Map.MaxY + Map.MinY) / 2) * Zoom);

      A2 := FindCity(Road[I].ToCity);
      X1 := Round((City[A2].X - (Map.MaxX + Map.MinX) / 2) * Zoom);
      Y1 := Round((City[A2].Y - (Map.MaxY + Map.MinY) / 2) * Zoom);

      MoveTo(X + CenterX, Y + CenterY);
      LineTo(X1 + CenterX, Y1 + CenterY);
    end;

    Pen.Color := clRed;

    for I := 0 to High(City) do
    begin
      X := Round((City[I].X - (Map.MaxX + Map.MinX) / 2) * Zoom);
      Y := Round((City[I].Y - (Map.MaxY + Map.MinY) / 2) * Zoom);

      Rectangle(Bounds(X - 2 + CenterX, Y - 2 + CenterY, 6, 6));
      Rectangle(Bounds(X + CenterX, Y + CenterY, 2, 2));

      TextOut(X + 8 + CenterX, Y + CenterY, City[I].Name);
    end;
  end;
end;

procedure TForm1.Cesty1Click(Sender: TObject);
begin
  (Sender as TMenuItem).Checked := not (Sender as TMenuItem).Checked;
  MapView.Invalidate;
end;

procedure TForm1.MapViewMouseMove(Sender: TObject; Shift: TShiftState; X,
  Y: Integer);
var I: Integer;
begin
  if not Loading then
  begin
    I := Map.GetCityFromPos(Round((X - CenterX) / Zoom + (Map.MaxX + Map.MinX) / 2),
                            Round((Y - CenterY) / Zoom + (Map.MaxY + Map.MinY) / 2));
    if I > -1 then StatusBar1.SimpleText := Map.City[I].Name
    else StatusBar1.SimpleText := 'Informaci o mst zjistte dvojklikem na nj.';
  end;
end;

procedure TForm1.MapViewMouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
var I, J: Integer;
    M: String;
begin
  if (not Loading) and (ssDouble in Shift) then
  begin
    I := Map.GetCityFromPos(Round((X - CenterX) / Zoom + (Map.MaxX + Map.MinX) / 2),
                            Round((Y - CenterY) / Zoom + (Map.MaxY + Map.MinY) / 2));
    if I > -1 then
    begin
      M :=Map.City[I].Name;
      StatusBar1.SimpleText := M;

      with Form2 do
      begin
        Caption := 'Informace o mst ' + M;

        StringGrid1.RowCount := 1;
        for J := 0 to High(Map.Road) do
        begin
          if AnsiSameText(Map.Road[J].FromCity, M) then
          begin
            StringGrid1.RowCount := Succ(StringGrid1.RowCount);
            StringGrid1.Cells[0, Pred(StringGrid1.RowCount)] := Map.Road[J].ToCity;
            StringGrid1.Cells[1, Pred(StringGrid1.RowCount)] := Format('%d km', [Map.Road[J].Length]);
            StringGrid1.Cells[2, Pred(StringGrid1.RowCount)] := TimeToStr(Map.Road[J].Length / Map.Road[J].AvSpeed / 24);
          end;
          if AnsiSameText(Map.Road[J].ToCity, M) then
          begin
            StringGrid1.RowCount := Succ(StringGrid1.RowCount);
            StringGrid1.Cells[0, Pred(StringGrid1.RowCount)] := Map.Road[J].FromCity;
            StringGrid1.Cells[1, Pred(StringGrid1.RowCount)] := Format('%d km', [Map.Road[J].Length]);
            StringGrid1.Cells[2, Pred(StringGrid1.RowCount)] := TimeToStr(Map.Road[J].Length / Map.Road[J].AvSpeed / 24);
          end;
        end;
        StringGrid1.FixedRows := 1;
      end;

      Form2.Show;
    end
    else StatusBar1.SimpleText := 'Informaci o mst zjistte dvojklikem na nj.';
  end;
end;

procedure TForm1.Vyhledatcestu1Click(Sender: TObject);
begin
  Form3.Show;
end;

initialization
  GetLocaleFormatSettings(LCID_INSTALLED, Settings);
  Settings.DecimalSeparator := '.';
  Zoom := 1;
  
end.
